/*********************************************************************************
**
**   $Id: DataForm.h,v 1.10 2004/05/04 01:05:00 croftj Exp $
**   Copyright (c) 2004 Joe Croft <joe@croftj.net>
**   
**   This file is part of ah2.
**
**   ah2 is free software; you can redistribute it and/or modify
**   it under the terms of the GNU General Public License as published by
**   the Free Software Foundation; either version 2 of the License, or
**   (at your option) any later version.
**
**   Foobar is distributed in the hope that it will be useful,
**   but WITHOUT ANY WARRANTY; without even the implied warranty of
**   MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
**   GNU General Public License for more details.
**
**   You should have received a copy of the GNU General Public License
**   along with Foobar; if not, write to the Free Software
**   Foundation, Inc., 59 Temple Place, Suite 330, Boston, MA  02111-1307  USA
**
*********************************************************************************/

#ifndef DATAFORM_H
#define DATAFORM_H

#include <qcombobox.h>
#include <qframe.h>
#include <qlayout.h>
#include <qlineedit.h>
#include <qsqlcursor.h>
#include <qsqldatabase.h>
#include <qsqleditorfactory.h>
#include <qsqlform.h>
#include <qsqlrecord.h>
#include <qstring.h>

struct DataFields 
{
   QString  dataName;
   QString  label;
   int      row;
   int      col;
   int      colSpan;
   int      rowSpan;
   int      minWidth;
   int      maxWidth;
   bool     ro;
   QString  defvalue;
   QWidget  *widget;
   QString  propName;
};

/*
class DataSqlEditorFactory : public QSqlEditorFactory
{
   Q_OBJECT

public:   
   QWidget *createEditor(QWidget *p, const QSqlField *f) 
   {
      QMap<QString, QWidget*>::Iterator it;
      if ( it = formWidgets->find(f->name()) != formWidgets.end() ) 
         return(it.data());
      else 
         return( QSqlEditorFactory::createEditor(p, f);
   }
};//
*/

class StringSelect : public QComboBox 
{
   Q_OBJECT               
   Q_PROPERTY( QString strsel READ text WRITE setText );

public:
   StringSelect(QString strsel, QWidget * parent = 0, const char * name = 0) : QComboBox(parent, name) { initialize(strsel); };
   QString text() const;
   void setText(QString t);

public slots:
   void initialize(QString s);

protected:

private:

};

class YesNoSelect : public QComboBox 
{
   Q_OBJECT               
   Q_PROPERTY( QString yesno READ text WRITE setText );

public:
   YesNoSelect(QWidget * parent = 0, const char * name = 0) : QComboBox(parent, name) { initialize(); };
   QString text() const;
   void setText(QString t);

public slots:
   void initialize();

protected:

private:

};

class MoneyEdit : public QLineEdit
{
   Q_OBJECT
   Q_PROPERTY( double money READ text WRITE setText );

public:
   MoneyEdit ( QWidget * parent, const char * name = 0 );
   MoneyEdit ( const QString & contents, QWidget * parent, const char * name = 0 );

   double text() const;
   static QString formatCurrency(double f);

public slots:
   void setText(const double);

private:
protected:
};

class DataForm : public QFrame 
{
   Q_OBJECT
   
public:
   DataForm(QSqlDatabase *db, const QString table, int rows, int cols, 
            QWidget *pParent = 0, const char *pName = 0, WFlags f = 0 );
   ~DataForm() {};
   void setFields(const DataFields *fields);
   QString getLastIdent();

public slots:   
   void refresh(QSqlRecord *);
   void refresh(QString item);
   void updateRecord();
   void insertRecord();
   void deleteRecord();
   void beginTransaction();
   void commitTransaction();
   void rollbackTransaction();
   void clearForm()
   {
      if ( sqlForm != 0 ) 
         sqlForm->clearValues(true);
   };

signals:
   void updated();


protected:
private:
   QMap<QString, QWidget*>    formFields;
   QMap<QString, QWidget*>    formWidgets;
   QMap<QString, QString>     widgetProperties;
   QMap<QString, QString>     fieldDefaults;
   QGridLayout                *gl;
   QString                    dbTable;
   QString                    ident;
   QString                    strIdent;
   QSqlForm                   *sqlForm;
   QSqlRecord                 *sqlRecord;
   QSqlCursor                 *sqlCursor;
   QSqlDatabase               *db;
   bool                       insertMode;
   bool                       inTransaction;

};

#endif


